from typing import Dict
import platform

IS_WINDOWS = platform.system() == "Windows"

if IS_WINDOWS:
    import psutil


def get_process_list() -> Dict:
    """Get process list.

    Returns dictionary mapping process id to process name
    """
    return {proc.pid: proc.name() for proc in psutil.process_iter()}


def is_numeric(value):
    try:
        float(value)
    except ValueError:
        return False
    else:
        return float(value).is_integer()



from pathlib import Path
from typing import Dict, List, Optional, Union

from RPA.core.windows.helpers import IS_WINDOWS, is_numeric
from RPA.core.windows.locators import MatchObject
from RPA.core.windows.window import WindowMethods

if IS_WINDOWS:
    import uiautomation as auto

    RecordElement = Dict[str, Optional[Union[float, str, auto.Control, List[str]]]]


class ElementInspector:
    """Element locator inspector"""

    @classmethod
    def inspect_element(
        cls,
        action: str = "Click",
        control_window: bool = True,
        recording: Optional[List["RecordElement"]] = None,
        verbose: bool = False,
    ) -> List[str]:
        """Inspect Windows element under mouse pointer.

        :param action: Action attached to the locator.
        :param control_window: Include relevant ``Control Window  ...`` statement or
            not.
        :param recording: Where to store records.
        :param verbose: Show exhaustive locators if `True`, otherwise just simple ones.
        """
        # TODO: Python syntax support too instead of just RF.
        output = []
        with auto.UIAutomationInitializerInThread(debug=False):
            control = auto.ControlFromCursor()
            parent_control = control.GetParentControl()
            try:
                top_level_control = control.GetTopLevelControl()
            except AttributeError:
                top_level_control = None
                top_level_name = top_level_handle = "N/A"
            else:
                top_level_name = top_level_control.Name
                top_level_handle = top_level_control.NativeWindowHandle
                try:
                    exec_path = WindowMethods.get_fullpath(top_level_control.ProcessId)
                except Exception:  # pylint: disable=broad-except
                    exec_path = ""

            top_properties = cls._get_element_key_properties(
                top_level_control, verbose=verbose
            )

            parent_properties = cls._get_element_key_properties(
                parent_control, verbose=verbose
            )
            child_properties = cls._get_element_key_properties(control, verbose=verbose)

            parent_locator = " and ".join(parent_properties) or "N/A"
            child_locator = " and ".join(child_properties) or "N/A"

            locator_path = f"{parent_locator} > {child_locator}"
            if "name:" in child_locator or "id:" in child_locator:
                locator_path = child_locator

            if control_window:
                output.append(f"Control Window  {top_level_name}")
            if action:
                output.append(f"{action}  {locator_path}")
            else:
                output.append(locator_path)

            if recording is not None:
                recording.append(
                    {
                        "type": "locator",
                        "exec_path": exec_path,
                        "exec": Path(exec_path).name,
                        "top": top_level_name,
                        "top_handle": top_level_handle,
                        "x": top_level_control,
                        "locator": locator_path,
                        "top_props": top_properties,
                        "parent_props": parent_properties,
                        "props": child_properties,
                        "name": parent_control.Name if parent_control else None,
                        "control": parent_control,
                    }
                )

        return output

    @staticmethod
    def _get_element_key_properties(
        element, *, verbose: bool, regex_limit: int = 300
    ) -> List[str]:
        if not element:
            print("Got null element!")
            return []

        name = element.Name.strip()
        automation_id = element.AutomationId
        control_type = element.ControlTypeName.strip()
        class_name = element.ClassName.strip()
        locators = []
        if len(name) > 0:
            name_property = "name:"
            if len(name) > regex_limit:
                name_property = "regex:"
                name = name[:regex_limit].strip()
            if " " in name:
                q = MatchObject.QUOTE
                name = f"{q}{name}{q}"
            locators.append(f"{name_property}{name}")
        if automation_id and not is_numeric(automation_id):
            locators.append(f"id:{automation_id}")
        if len(control_type) > 0:
            locators.append(f"type:{control_type}")
        if len(class_name) > 0:
            locators.append(f"class:{class_name}")
        if locators:
            if not verbose:
                locators = locators[:1]
            return locators

        print("Was unable to construct locator for the control!")
        return []



import contextlib
import logging
import os
import platform
import stat
from pathlib import Path
from typing import Dict, List, Optional

import requests
from packaging import version
from requests import Response
from selenium import webdriver
from selenium.webdriver.common.service import Service
from selenium.webdriver.remote.webdriver import WebDriver
from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.core.download_manager import WDMDownloadManager
from webdriver_manager.core.http import WDMHttpClient
from webdriver_manager.core.manager import DriverManager
from webdriver_manager.core.utils import os_name as get_os_name
from webdriver_manager.firefox import GeckoDriverManager
from webdriver_manager.microsoft import EdgeChromiumDriverManager, IEDriverManager
from webdriver_manager.opera import OperaDriverManager

from RPA.core.robocorp import robocorp_home


LOGGER = logging.getLogger(__name__)

DRIVER_ROOT = robocorp_home() / "webdrivers"
AVAILABLE_DRIVERS = {
    # Driver names taken from `webdrivermanager` and adapted to `webdriver_manager`.
    "chrome": ChromeDriverManager,
    "firefox": GeckoDriverManager,
    "gecko": GeckoDriverManager,
    "mozilla": GeckoDriverManager,
    # NOTE: Selenium 4 dropped support for Opera.
    #  (https://github.com/SeleniumHQ/selenium/issues/10835)
    "opera": OperaDriverManager,
    # NOTE: In Selenium 4 `Edge` is the same with `ChromiumEdge`.
    "edge": EdgeChromiumDriverManager,
    "chromiumedge": EdgeChromiumDriverManager,
    # NOTE: IE is discontinued and not supported/encouraged anymore.
    "ie": IEDriverManager,
}
_DRIVER_PREFERENCE = {
    "Windows": ["Chrome", "Firefox", "ChromiumEdge"],
    "Linux": ["Chrome", "Firefox", "ChromiumEdge"],
    "Darwin": ["Chrome", "Firefox", "ChromiumEdge", "Safari"],
    "default": ["Chrome", "Firefox"],
}


def _get_browser_order_from_env() -> Optional[List[str]]:
    browsers: str = os.getenv("RPA_SELENIUM_BROWSER_ORDER", "")
    if browsers:
        return [browser.strip() for browser in browsers.split(sep=",")]

    return None  # meaning there's no env var to control the order


def get_browser_order() -> List[str]:
    """Get a list of preferred browsers based on the environment variable
    `RPA_SELENIUM_BROWSER_ORDER` if set.

    The OS dictates the order if no such env var is set.
    """
    browsers: Optional[List[str]] = _get_browser_order_from_env()
    if browsers:
        return browsers

    return _DRIVER_PREFERENCE.get(platform.system(), _DRIVER_PREFERENCE["default"])


def _set_driver_preference() -> Dict[str, List[str]]:
    pref = _DRIVER_PREFERENCE.copy()
    browsers: Optional[List[str]] = _get_browser_order_from_env()
    if browsers:
        for op_sys in pref:
            pref[op_sys] = browsers
    return pref


# FIXME(cmin764): This constant is deprecated and is planned for removal in the next
#  major upgrade. (use `get_browser_order` function instead)
DRIVER_PREFERENCE = _set_driver_preference()


class Downloader(WDMHttpClient):

    """Custom downloader which disables download progress reporting."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.driver = None

    def _fix_mac_arm_url(self, url) -> str:
        if "m1" not in self.driver.get_os_type():
            return url

        # FIXME(cmin764): Remove this when the issue below gets closed
        #  https://github.com/SergeyPirogov/webdriver_manager/issues/446
        browser_version = self.driver.get_version()
        if version.parse(browser_version) >= version.parse("106.0.5249.61"):
            url = url.replace("mac64_m1", "mac_arm64")
        return url

    def get(self, url, **kwargs) -> Response:
        url = self._fix_mac_arm_url(url)
        resp = requests.get(url=url, verify=self._ssl_verify, stream=True, **kwargs)
        self.validate_response(resp)
        return resp


@contextlib.contextmanager
def suppress_logging():
    """Suppress webdriver-manager logging."""
    wdm_log = "WDM_LOG"
    original_value = os.getenv(wdm_log, "")
    try:
        os.environ[wdm_log] = str(logging.NOTSET)
        yield
    finally:
        os.environ[wdm_log] = original_value


def start(browser: str, service: Optional[Service] = None, **options) -> WebDriver:
    """Start a webdriver with the given options."""
    browser = browser.strip()
    webdriver_factory = getattr(webdriver, browser, None)
    if not webdriver_factory:
        raise ValueError(f"Unsupported browser: {browser}")

    # NOTE: It is recommended to pass a `service` rather than deprecated `options`.
    driver = webdriver_factory(service=service, **options)
    return driver


def _to_manager(browser: str, root: Path = DRIVER_ROOT) -> DriverManager:
    browser = browser.strip()
    manager_factory = AVAILABLE_DRIVERS.get(browser.lower())
    if not manager_factory:
        raise ValueError(
            f"Unsupported browser {browser!r}! (choose from: {list(AVAILABLE_DRIVERS)})"
        )

    downloader = Downloader()
    download_manager = WDMDownloadManager(downloader)
    manager = manager_factory(path=str(root), download_manager=download_manager)
    downloader.driver = manager.driver
    return manager


def _set_executable(path: str) -> None:
    st = os.stat(path)
    os.chmod(
        path,
        st.st_mode | stat.S_IXOTH | stat.S_IXGRP | stat.S_IEXEC,
    )


def download(browser: str, root: Path = DRIVER_ROOT) -> Optional[str]:
    """Download a webdriver binary for the given browser and return the path to it."""
    manager = _to_manager(browser, root)
    driver = manager.driver
    resolved_os = getattr(driver, "os_type", driver.get_os_type())
    os_name = get_os_name()
    if os_name.lower() not in resolved_os.lower():
        LOGGER.warning(
            "Attempting to download incompatible driver for OS %r on OS %r! Skip",
            resolved_os,
            os_name,
        )
        return None  # incompatible driver download attempt

    with suppress_logging():
        path: str = manager.install()
    if platform.system() != "Windows":
        _set_executable(path)
    LOGGER.debug("Downloaded webdriver to: %s", path)
    return path



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['RPA', 'RPA.core', 'RPA.core.locators', 'RPA.core.vendor', 'RPA.core.windows']

package_data = \
{'': ['*']}

install_requires = \
['packaging>=21.3,<22.0',
 'pillow>=9.1.1,<10.0.0',
 'selenium>=4.4.0,<5.0.0',
 'webdriver-manager>=3.8.3,<4.0.0']

extras_require = \
{':python_full_version != "3.7.6" and python_full_version != "3.8.1" and sys_platform == "win32"': ['pywin32>=300,<304'],
 ':sys_platform == "win32"': ['psutil>=5.9.0,<6.0.0',
                              'uiautomation>=2.0.15,<3.0.0']}

setup_kwargs = {
    'name': 'rpaframework-core',
    'version': '10.4.0',
    'description': 'Core utilities used by RPA Framework',
    'long_description': 'rpaframework-core\n=================\n\nThis package is a set of core functionality and utilities used\nby `RPA Framework`_. It is not intended to be installed directly, but\nas a dependency to other projects.\n\n.. _RPA Framework: https://rpaframework.org\n',
    'author': 'RPA Framework',
    'author_email': 'rpafw@robocorp.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://rpaframework.org/',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
